@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# const.py - A set of structures and constants used to implement the Ethernet/IP protocol
-#
-# Copyright (c) 2020 Ian Ottoway <ian@ottoway.dev>
+# Copyright (c) 2021 Ian Ottoway <ian@ottoway.dev>
 # Copyright (c) 2014 Agostino Ruscito <ruscito@gmail.com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -29,17 +27,23 @@ from typing import NamedTuple, Any, Optional
 from reprlib import repr as _r
 
 
+__all__ = ["Tag"]
+
+
 class Tag(NamedTuple):
-    tag: str
-    value: Any
-    type: Optional[str] = None
-    error: Optional[str] = None
+    tag: str  #: tag name of tag read/written or request name (generic message)
+    value: Any  #: value read/written, may be ``None`` on error
+    type: Optional[str] = None  #: data type of tag
+    error: Optional[str] = None  #: error message if unsuccessful, else ``None``
 
     def __bool__(self):
+        """
+        ``True`` if both ``value`` is not ``None`` and ``error`` is ``None``, ``False`` otherwise
+        """
         return self.value is not None and self.error is None
 
     def __str__(self):
-        return f'{self.tag}, {_r(self.value)}, {self.type}, {self.error}'
+        return f"{self.tag}, {_r(self.value)}, {self.type}, {self.error}"
 
     def __repr__(self):
         return f"{self.__class__.__name__}(tag={self.tag!r}, value={self.value!r}, type={self.type!r}, error={self.error!r})"
